<?php

namespace Drupal\aggrid\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\aggrid\Entity;
use Drupal\Core\Form\ConfigFormBase;

/**
 * Plugin implementation of the 'aggrid_formatter_type' formatter.
 *
 * @FieldFormatter(
 *   id = "html_formatter_type",
 *   label = @Translation("HTML grid view mode"),
 *   field_types = {
 *     "aggrid"
 *   }
 * )
 */
class htmlFormatterType extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        // Implement default settings.
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return [
        // Implement settings form.
      ] + parent::settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    // Implement settings summary.

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      $config = \Drupal::config('aggrid.general');
      $field_settings = $this->getFieldSettings();

      $field_name = $this->fieldDefinition->getName();

      $item_id = Html::getUniqueId("ht-$field_name-$delta");

      $aggridEntity = Entity\aggrid::load($items[$delta]->aggrid_id);
      $aggridDefault = json_decode($aggridEntity->get('aggridDefault'));

      if ($items[$delta]->value != '') {
        $aggridValue = json_decode($items[$delta]->value);
      } else {
        $aggridValue = $aggridDefault->rowData;
      }

      $columns = [];
      $rowData = $aggridValue;

      // Loop through header array and dive down max 3 header rows. Squash all down to single row with only the items with fields.
      // Header 1
      foreach($aggridDefault->columnDefs as $column) {
        if (isset($column->children)) { // If children, then dive down for headers, otherwise establish column
          foreach ($column->children as $child) {
            // Header 2
            if (isset($child->children)) {
              foreach ($child->children as $subchild) {
                if (isset($subchild->field)) {
                  // Header from row 3
                  $columns[$column->headerName . ' - ' . $child->headerName . ' - ' . $subchild->headerName] = [];
                  $columns[$column->headerName . ' - ' . $child->headerName . ' - ' . $subchild->headerName]['field'] = $subchild->field;
                }
              }
            } else {
              if (isset($child->field)) {
                // Header from row 2
                $columns[$column->headerName . ' - ' . $child->headerName] = [];
                $columns[$column->headerName . ' - ' . $child->headerName]['field'] = $child->field;
              }
            }
          }
      } else {
          if (isset($column->field)) {
            // Header from row 1
            $columns[$column->headerName] = [];
            $columns[$column->headerName]['field'] = $column->field;
          }
        }
      }

      // Build table
      $table_render = '';
      $table_render .= '<table><thead><tr>';

      // Loop columns and apply header names
      foreach($columns as $column => $value) {
        $table_render .= '<th>'. $column .'</th>';
      }

      $table_render .= '</tr></thead><tbody>';

      for ($i = 0; $i < count($rowData); $i++) {
        $table_render .= '<tr>';

        foreach($columns as $column => $value) {
          $colField = $columns[$column]['field'];
          $table_render .= '<td>'. $rowData[$i]->$colField .'</td>';
        }

        $table_render .= '</tr>';
      }

      $table_render .= '</tbody></table>';

      $elements[$delta]['container'] = [
        '#suffix' => $table_render,
        '#attributes' => ['id' => [$item_id .'_table']],
      ];
    }

    return $elements;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {
    // The text value has no text format assigned to it, so the user input
    // should equal the output, including newlines.
    return nl2br(Html::escape($item->value));
  }

}
